#!/bin/sh

# Skrypt tworzy skojarzenia plikow dla PEM-Heart w oparciu o dane z biezacego katalogu
# Skrypt powinien być umieszczony w katalogu aplikacji, której skojarzenia mają być ustawione
# Zakłada obecność plików konfiguracyjnych umieszczonych w podkatalogu data lub wskazanym
# w zmiennej konfiguracyjnej na początku

APPNAME="@APPNAME@"
VERSION="@VERSION@"

# Te mogą być względne, jeśli tak jest, to muszą być w katalogu z tym skryptem.
DATADIR="@DATADIR@"
ICONDIR="@ICONDIR@"

argv0="$0"
argv0dir=`dirname "$argv0"`


iconv=`which iconv 2>/dev/null`
if [ "X$iconv" != "X" ]; then
  lc=`locale | grep LANG`
  case "$lc" in
    *UTF-8*) iconv=cat;;
    *.*)     iconv="iconv -f UTF-8 -t `echo $lc|sed -n -e 's/.*\.\(.*\)/\1/p'`";;
    *)       iconv="iconv -f UTF-8 -t ISO8859-2";;
  esac
else
  iconv=cat
fi



detect_lang() {
  selected_lang=""
  case $LANG in
    '')   ;;
    pl*)  selected_lang=pl;;
      *)  selected_lang=en;;
  esac

  if [ "X$selected_lang" = "X" ] ;then
    case $LC_ALL in
      pl*)  selected_lang=pl;;
    esac
  fi

  test "X$selected_lang" = "X" && selected_lang=en
  echo $selected_lang
}


echo_cp() {
  echo "$@" | $iconv
}


print_error() {

  echo "$@" > /dev/stderr
}



print_version() {
  a="$APPNAME"
  v="$VERSION"
  uv_pl="nieznana wersja"
  uv_en="unspecified version"
  
  test "X$v" == "X" -o "X$v" == "X@VERSION@" && eval v="\$uv_`detect_lang`"
  test "X$a" == "X" -o "X$a" == "X@APPNAME@" && a="PEM-Heart"
  echo_cp "$a $v"
  echo_cp "(C) Enigma Systemy Ochrony Informacji Sp. z o.o."
}



# Normalizes path - prints its absolute value
# Assumes path exists
absolute_path() {
  if [ ! -e "$1" ]; then
    print_error "$1: does not exist"
    exit 1;
  fi

  if [ -d "$1" ]; then
    cd "$1" && pwd

  else
    b=`basename $1`
    d=`dirname $1`
    echo `cd "$d" && pwd`/$b

  fi
}


print_help_en() {
  echo "$0:"
  echo "   Associates $APPNAME with supported file types or removes this association"
  echo ""
  echo "Usage: $0 <action> [options]"
  echo ""
  echo "Actions:"
  echo "  --install       installs associations"
  echo "  --install-only  installs only selected associations"
  echo "  --remove        removes associations"
  echo "  --remove-only   removes only selected associations"
  echo "  --version       displays application version"
  echo "  --help          this screen"
  echo ""
  echo "Options:"
  echo "  --user          force install/remove in user context"
  echo "  --system        force install/remove in system context"
  echo "       Default installation is performed in system context if this script"
  echo "       is executed from root account, otherwise is performed in user context."
  echo "  --menu          force install/remove application menu items"
  echo "  --icons         force install/remove any icons / an or images"
  echo "  --mime-types    force install/remove application mime types"
  echo "  --mime-icons    force install/remove icons for mime types"
  echo "  --ctx-menu      force install/remove context menu items"
  echo "                  for supported Desktop Environments (DE)"
  echo "  --no-menu       don't install/remove application menu items"
  echo "  --no-icons      don't install/remove any icons / an or images"
  echo "  --no-mime-types don't install/remove application mime types"
  echo "  --no-mime-icons don't install/remove mime type icons"
  echo "  --no-ctx-menu   don't install/remove context menu items"
  echo ""
  echo "  --force-<DE>    forces installation/remove associations for selected DE"
  echo "  --no-<DE>       don't install/remove associations for selected DE"
  echo "  --auto-<DE>     aumomatically select installation/removal of DE (default)"
  echo "  Supported DE (Desktop Environments): kde, gnome, xfce, lxde"
  echo ""
  echo "Other options:"
  echo "  --verbose       be verbose on performed operations"
  echo "  --quiet         don't display any messages"
  echo "  --disable-xdg   don't use xdg commands to manage application desktop database,"
  echo "                  manualy copy all files to required locations instead"
  echo ""
}

print_help_pl() {
  echo_cp "$0:"
  echo_cp "   Tworzy lub usuwa skojarzenia dla programu $APPNAME dla wspieranych typów plików"
  echo_cp ""
  echo_cp "Sposób użycia: $0 <akcja> [opcje]"
  echo_cp ""
  echo_cp "Akcje:"
  echo_cp ""
  echo_cp "  --install       instalacja skojarzeń"
  echo_cp "  --install-only  instalacja tylko wybranych skojarzeń"
  echo_cp "  --remove        usuwa skojarzenia"
  echo_cp "  --remove-only   usuwa tylko wskazane skojarzenia"
  echo_cp "  --version       wyświetla wersję aplikacji"
  echo_cp "  --help          ten ekran"
  echo_cp ""
  echo_cp "Opcje:"
  echo_cp "  --user          wymuś instalację/usunięcie dla bieżącego użytkownika"
  echo_cp "  --system        wymuś instalację/usunięcie dla całego systemu"
  echo_cp "      Domyślnie instalacja/usunięcie jest wykonywana dla systemu, jeśli skrypt jest"
  echo_cp "      uruchomiany z konta root, w przeciwnym przypadku dla użytkownika"
  echo_cp "  --menu          wymuś instalację/usunięcie elementów menu głównego"
  echo_cp "  --icons         wymuś instalację/usunięcie ikon i obrazków programu"
  echo_cp "  --mime-types    wymuś instalację/usunięcie typów mime"
  echo_cp "  --mime-icons    wymuś instalację/usunięcie ikon dla typów mime"
  echo_cp "  --ctx-menu      wymuś instalację/usunięcie menu kontekstowego dla"
  echo_cp "                  wspieranych typów Pulpitu (DE)"
  echo_cp "  --no-menu       nie instaluj/usuwaj elementów głównego menu"
  echo_cp "  --no-icons      nie instaluj/usuwaj żadnych ikon ani obrazków"
  echo_cp "  --no-mime-types nie instaluj/usuwaj żadnych typów mime"
  echo_cp "  --no-mime-icons nie instaluj/usuwaj ikon dla typów mime"
  echo_cp "  --no-ctx-menu   nie instaluj/usuwaj żadnych elementów menu kontekstowego"
  echo_cp ""
  echo_cp "  --force-<DE>    wymusza instalację/usunięcie elementów menu kontekstowego"
  echo_cp "                  dla wskazanego Pulpitu (DE, Desktop Environment) "
  echo_cp "  --no-<DE>       nie instaluj/usuwaj skojarzeń dla wskazanego DE"
  echo_cp "  --auto-<DE>     automatycznie instaluj/usuwaj instalację dla DE (domyślnie)"
  echo_cp "  Obsługiwane typy Pulpitu (DE): kde, gnome, xfce, lxde"
  echo_cp ""
  echo_cp "Other options:"
  echo_cp "  --verbose       wyświetl komunikaty diagnostyczne"
  echo_cp "  --quiet         nie wyświetlaj żadnych komunikatów"
  echo_cp "  --disable-xdg   nie używaj poleceń xdg do zarządzania, tylko bezpośrednio skopiuj"
  echo_cp "                  wymagane pliki w odpowiednie lokalizacje"
  echo_cp ""
}

print_help() {

  print_help_`detect_lang`
}


search_existing_dir_in_path() {
  old_IFS=$IFS
  IFS=':'
  for p in $1; do
    if [ -d $p ]; then
      echo $p
    fi
  done

  IFS="$old_IFS"
}

print_first_path_on_list() {
  old_IFS=$IFS
  IFS=':'
  set $1
  echo $1
  IFS="$old_IFS"
}

print_last_path_on_list() {
  old_IFS=$IFS
  IFS=':'
  dir=
  for p in $1; do
    if [ -d $p ]; then
      dir="$p"
    fi
  done
  echo $dir
  IFS="$old_IFS"
}

split_list() {
  old_IFS=$IFS
  IFS=':'
  set "$@"
  IFS="$old_IFS"  
  
  echo "$@"
}

detect_mode() {
  test "X$1" = "X" && set `id -u`
  case "$1" in
    0|root|sys*|global) echo system;;
    *)                  echo user;; 
  esac  
}

# Finds first existing or just first directory from one of two lists (user/system), depending on first argument
get_dir_based_on_context() {
  # $1 - context
  # $2 - systemdirectory lists
  # $3 - user directory list
  dir=
  case "$1" in
    0|root|sys*|global)
      if [ "X$2" != "X" ]; then
        dir=`search_existing_dir_in_path "$2"`
        test "X$dir" = "X" && dir=`print_first_path_on_list "$2"`
      fi ;;
    *)
      if [ "X$3" != "X" ]; then
        dir=`search_existing_dir_in_path "$3"`
        test "X$dir" = "X" && dir=`print_first_path_on_list "$3"`
      fi ;;
  esac
  
  echo $dir
}

detect_app() {
  local appname="$1"
  shift
  app=`which "$appname" 2>/dev/null`
  test "X$app" != "X"
  local r=$?
  local p=
  test $r = 0 && p=1 || p=0
  
  for arg; do
    case "$1" in
      --print)          echo $p;;
      --print-verbose)  echo "$appname: $p";;
      --true)           r=0;;
    esac
  done
  return $r
}

detect_kde() {
  detect_app kde4-config "$@"
}

detect_gnome() {
  detect_app gnome-session "$@"
}

detect_xfce() {
  detect_app xfce4-session "$@"
}

detect_lxde() {
  detect_app lxsession "$@"
}


detect_kde_fm() {
  detect_args=
  if [ "X$1" = "X--print-verbose" ]; then
    detect_args=$1
    shift
  fi
  
  detect_app konqueror $detect_args || detect_app dolphin $detect_args
  r=$?
  p=
  test $r = 0 && p=1 || p=0

  for arg; do
    case "$1" in
      --print)   echo $p;;
      --true)    r=0;;
    esac
  done
  return $r
}

detect_gnome_fm() {
  detect_app nautilus "$@"
}

detect_xfce_fm() {
  detect_app thunar "$@"
}

detect_lxde_fm() {
    detect_app pcmanfm "$@"
}



# Menu item actions

print_desktop_field() {
  # $1 desktop file
  # $2 field name
  # $3 noquotes
  local lang=`detect_lang`
  local sed_expr='s/^.*= *\(.*\)/"\1"/'
  test "X$3" = "X1" && sed_expr='s/^.*= *\(.*\)/\1/'
  
  value=`cat "$1" 2>/dev/null | grep "^$2\[$lang\] *=" | sed "$sed_expr"`
  test "X$value" = "X" && value=`cat "$1" 2>/dev/null | grep "^$2 *=" | sed "$sed_expr"`
  
  echo $value
}

print_desktop_entry_name() {
  
  name=`print_desktop_field "$1" Name`
  if [ "X$name" = "X" ]; then
    basename $1
  else
    echo $name
  fi
}

print_kde_actions_names() {

  name=`print_desktop_field "$1" Name`
  if [ "X$name" = "X" ]; then
    basename $1
  else
    menuname=`print_desktop_field "$1" "X-KDE-Submenu" 1`
    if [ "X$menuname" = "X" ]; then
      echo $name
    else
      echo $menuname ">" $name
    fi
  fi  
  
}

sys_menudir=/usr/share/applications
user_menudir=~/.local/share/applications

install_menu_items() {
  # $1 - context (user/system)
  # $2... source files and dirs
  local install_mode="$1"
  shift
  
  print_verbose 4 "Installing [mode=$install_mode] menu items: $@"
  
  # if no source files, exit with success
  test "X$1" = "X" && return 0

  xdg_desktop_menu=
  test $disable_xdg = 1 || xdg_desktop_menu=`which xdg-desktop-menu 2>/dev/null`
  if [ "X$xdg_desktop_menu" = "X" ]; then
    # no xdg-desktop-menu - manually copy desktop files to destination dir
    dir=`get_dir_based_on_context "$install_mode" "$sys_menudir" "$user_menudir"`
    print_verbose 2 "XDG menu support disabled - manually copying desktop files to $dir"
    
    mkdir -p "$dir"
    for f; do
      bf=`basename $f`
      case "$f" in
        *.desktop|*.directory)
           name=`print_desktop_entry_name "$f"`
           print_verbose 1 "Installing menu item $name"
           print_verbose 2 "Installing menu file $dir/$bf"
           cp "$f" "$dir"
           ;;
        *)
          test -f $f && print_verbose 1 "Skipping file $bf - does not end with .desktop"
          ;;
       esac
    done
  
    if type update-desktop-database 2>/dev/null >/dev/null; then
      print_verbose 2 "Updating cache"
      update-desktop-database "$dir"
    else
      print_verbose 1 "No menu cache updated. You may need to log-out to see new menu entries"
    fi
  else
    for f; do
      bf=`basename $f`
      case "$f" in
        *.desktop|*.directory)
          name=`print_desktop_field "$f" Name`
           name=`print_desktop_entry_name "$f"`
           print_verbose 1 "Installing menu item $name"
           print_verbose 2 "XDG [$install_mode]: installing menu item $bf"
          "$xdg_desktop_menu" install --noupdate --novendor --mode $install_mode "$f"
          ;;
          
        *)
          test -f $f && print_verbose 1 "XDG [$install_mode]: Skipping file $bf - does not end with .desktop"
          ;;
      esac
    done
    print_verbose 2 "Updating cache"
    "$xdg_desktop_menu" forceupdate --mode $install_mode
  fi
}

remove_menu_items() {
  # $1 - context (user/system)
  # $2... source files and dirs
  local install_mode="$1"
  shift
  
  print_verbose 4 "Removing  [mode=$install_mode] menu items: $@"

# if no source files, exit with success
  test "X$1" = "X" && return 0

  xdg_desktop_menu=
  test $disable_xdg = 1 || xdg_desktop_menu=`which xdg-desktop-menu 2>/dev/null`
  if [ "X$xdg_desktop_menu" = "X" ]; then
    # no xdg-desktop-menu - manually removing desktop files to destination dir
    local dir=`get_dir_based_on_context "$install_mode" "$sys_menudir" "$user_menudir"`
    print_verbose 2 "XDG menu support disabled - manually removing desktop files from $dir"
    local deleted=0
    for f; do
      bf=`basename $f`
      if [ -f "$dir/$bf" ]; then
         name=`print_desktop_entry_name "$f"`
         print_verbose 1 "Removing menu item $name"
         print_verbose 2 "Removing menu file $dir/$bf"
         rm -f "$dir/$bf"
         deleted=1
      fi
    done
  
    if [ $deleted = 1 ]; then
      if type update-desktop-database 2>/dev/null >/dev/null; then
        print_verbose 2 "Updating menu cache"
        update-desktop-database "$dir"
      else
        print_verbose 1 "No menu cache updated."
      fi
    fi
  else
    local deleted=0
    for f; do
      case $f in
        *.desktop|*.directory)
          if test -f $f; then
            bf=`basename $f`
            name=`print_desktop_entry_name "$f"`
            print_verbose 1 "Removing menu item $name"
            print_verbose 2 "XDG [$install_mode]: removing menu item $bf"
            "$xdg_desktop_menu" uninstall --noupdate --mode $install_mode "$f"
            deleted=1
          fi
          ;;
      esac
    done
    if test $deleted = 1; then
      print_verbose 2 "XDG [$install_mode]: updating menu cache"
      "$xdg_desktop_menu" forceupdate --mode $install_mode
    fi
  fi
}


sys_mimetypesdir=/usr/share/mime
user_mimetypesdir=~/.local/share/mime

install_update_mime_database() {
 
  local dbupdated=0
  if type update-mime-database 2>/dev/null >/dev/null; then
    print_verbose 2 "Updating mime cache"
    for dir; do
      update-mime-database "$dir"
      dbupdated=1
    done
  fi
  if type kbuildsycoca 2>/dev/null >/dev/null; then
    print_verbose 2 "Updating KDE mime cache"
    kbuildsycoca 2>/dev/null
    dbupdated=1
  fi
  if type kbuildsycoca4 2>/dev/null >/dev/null; then
    print_verbose 2 "Updating KDE4 mime cache"
    kbuildsycoca4 2>/dev/null
    dbupdated=1
  fi
    
  if test $dbupdated = 0; then
    print_verbose 1 "No mime-type cache updated. You may need to log-out to see new associations and icons"
  fi
}

install_mime_types() {
  # $1 - context (user/system)
  # $2... source files and dirs
  local install_mode="$1"
  shift
  
  print_verbose 4 "Installing [mode=$install_mode] mime types: $@"

# if no source files, exit with success
  test "X$2" = "X" && return 0

  # We'll need this dir for update mime database
  local dir=`get_dir_based_on_context "$install_mode" "$sys_mimetypesdir" "$user_mimetypesdir"`

  xdg_mime=
  test $disable_xdg = 1 || xdg_mime=`which xdg-mime 2>/dev/null`
  if [ "X$xdg_mime" = "X" ]; then
    # no xdg-mime - manually copy xml files to destination dir
    print_verbose 2 "XDG mime-type support disabled - manually copying mime-type definitions to $dir"
    
    mkdir -p "$dir" "$dir/packages"
    for f; do
      local bf=`basename $f`
      case "$f" in
        *.xml)
           print_verbose 1 "Installing mime-types: " `extract_mime_types "$f"`
           print_verbose 2 "Installing mime-info $dir/packages/$bf"
           cp "$f" "$dir/packages"
           ;;
        *)
          test -f $f && print_verbose 1 "Skipping file $bf - does not end with .xml"
          ;;
       esac
    done
  
  else
    for f; do
      local bf=`basename $f`
      case "$f" in
        *.xml)
          print_verbose 1 "Installing mime-types: "`extract_mime_types "$f"`
          print_verbose 2 "XDG [$install_mode]: installing mime-info $bf"
          "$xdg_mime" install --novendor --mode $install_mode "$f"
          ;;
          
        *)
          test -f $f && print_verbose 1 "XDG [$install_mode]: Skipping file $bf - does not end with .xml"
          ;;
      esac
    done
  fi

  install_update_mime_database "$dir"
}


remove_mime_types() {
  # $1 - context (user/system)
  # $2... source files and dirs
  local install_mode="$1"
  shift
  
  print_verbose 4 "Removing [mode=$install_mode] mime types: $@"

# if no source files, exit with success
  test "X$2" = "X" && return 0

  # We'll need this dir for update mime database
  local dir=`get_dir_based_on_context "$install_mode" "$sys_mimetypesdir" "$user_mimetypesdir"`

  xdg_mime=
  test $disable_xdg = 1 || xdg_mime=`which xdg-mime 2>/dev/null`
  if [ "X$xdg_mime" = "X" ]; then
    # no xdg-mime - manually copy xml files to destination dir
    
    if [ -d "$dir/packages" ]; then
      print_verbose 2 "XDG mime-type support disabled - manually removing mime-type definitions from $dir"

      for f; do
        local bf=`basename $f`
        if [ -f "$dir/packages/$bf" ]; then
          print_verbose 1 "Removing mime-types: "`extract_mime_types "$dir/packages/$bf"`
          print_verbose 2 "Removing mime-info $dir/packages/$bf"
          rm -f "$dir/packages/$bf"
        fi
      done
    fi
  else
    for f; do
      if test -f "$f"; then
        local bf=`basename $f`
        print_verbose 1 "Removing mime-types: "`extract_mime_types "$f"`
        print_verbose 2 "XDG [$install_mode]: removing mime-info $bf"
        "$xdg_mime" uninstall --mode $install_mode "$f"
      fi
    done
  fi

  install_update_mime_database "$dir"
}



extract_mime_types() {
    #1... - xml input file(s)
    cat "$@" 2>/dev/null | sed -n -e '/<mime-type type=".*"/p' | sed -n -e 's/^.*type="\([^"]*\).*$/\1/p'
}

mime_type_to_icon_name() {
    for mt; do
        echo "$mt" | sed -n -e 's|/|-|p'
    done
}


sys_iconsdir=/usr/share/icons
sys_pixmapsdir=/usr/share/pixmaps
user_iconsdir=~/.local/share/icons
user_pixmapsdir=~/.local/share/pixmaps:~/.kde/share/pixmaps

detect_kde && user_pixmapsdir=`print_first_path_on_list \`kde4-config --path pixmap\``

install_icons() {
  # $1 - mode (user/system)
  # $2 - context (mimetypes, apps)
  # $3 - application (icon) name
  # $4 - main image (largest possible).
  # $5... other image files, if present and contain main image base name and numeric suffix,
  #      will be used for specified size.
  # install for sizes 16 22 24 32 48 64 128
  # if ImageMagick's convert command is available, missing images will be converted, otherwise 
  # only images given in parameters will be installed
  set -e

  local install_mode="$1"
  local install_context="$2"
  local icon_name="$3"
  local main_image="$4"
  shift 4

  print_verbose 4 "Installing [mode=$install_mode, name=$icon_name] $install_context icons: $main_image $@"

  local tmp_dir=/tmp/tmp_icon_install.$$
  local imglist=
  local largest_size=128
  
  # if no images, exit with success
  test "X$main_image" = "X" && return 0
  
  local convert=`which convert 2>/dev/null`
  local main_image_base=
  case "$main_image" in
    *.png)  main_image_base=`basename "$main_image" .png`;;
    *.PNG)  main_image_base=`basename "$main_image" .PNG`;;
    *.Png)  main_image_base=`basename "$main_image" .Png`;;
    *.xpm)  main_image_base=`basename "$main_image" .xpm`;;
    *.XPM)  main_image_base=`basename "$main_image" .XPM`;;
    *.Xpm)  main_image_base=`basename "$main_image" .Xpm`;;
    *)
       # cut any extension
       main_image_base=`basename "$main_image"`
       main_image_base=`echo $main_image | sed 's/\..*//'`
       ;;
  esac
  
  if [ "X$convert" != "X" ]; then
    # check largest available size
    largest_size=`$convert -identify "$main_image" /dev/null | cut -d ' ' -f 3-3 | cut -d x -f1-1`
    test "X$largest_size" = "X" && largest_size=128
  fi
  
  # Gather all required images, if no size is available and 'convert' is available, use it
  # Copy all images to /tmp directory
  for size in 16 22 24 32 48 64 128; do
    local img_sz_src=
    for img; do
      case $img in
        *$size.[pP][nN][gG]|*$size.[xX][pP][mP])
          if test -f $img; then 
            local imgbase=`basename $img | sed "s/$size....//"`
            imgbase=`basename $imgbase -`
            imgbase=`basename $imgbase _`
            if [ "$imgbase" = "$icon_name" -o "$imgbase" = "$main_image_base" ]; then
              img_sz_src=$img
            fi
          fi
          ;;
      esac
    done
    
    if [ "X$img_sz_src" = "X" -a "X$convert" != "X" ]; then
      if [ "X$largest_size" = "X" -o "$size" -le "$largest_size" ]; then
        mkdir -p $tmp_dir
        print_verbose 3 "Resizing image $main_image_base-$size.png"
        $convert $img -resize ${size}x${size} "$tmp_dir/$main_image_base-$size.png"
        img_sz_src="$tmp_dir/$main_image_base-$size.png"
      fi
    fi
    
    test -n "$img_sz_src" && imglist="$imglist $size:$img_sz_src"
  done
  
  test "X$imglist" = "X" && imglist="32:$main_image"
  
  
  xdg_desktop_icon=
  test $disable_xdg = 1 || xdg_desktop_icon=`which xdg-icon-resource 2>/dev/null`
  if [ "X$xdg_desktop_icon" = "X" ]; then
    # no xdg-icon-resource - manually copy desktop files to destination dir
    local dir=`get_dir_based_on_context "$install_mode" "$sys_iconsdir" "$user_iconsdir"`
    local ctxdir="$dir/hicolor/$install_context"
    print_verbose 2 "XDG menu support disabled - manually copying icons files to $ctxdir"
    mkdir -p "$ctxdir"

    for l in $imglist; do
      local size=`echo "$l" | cut -d ':' -f 1-1`
      local f=`echo "$l" | cut -d ':' -f 2-`
      local bf=`basename "$f"`
      local dbf=
      case $f in
        *[pP][nN][gG])  dbf="$icon_name.png";;
        *[xX][pP][mM])  dbf="$icon_name.xpm";;
      esac
      
      if [ "X$dbf" = "X" ]; then
        print_verbose 1 "Skipping file $f - is not PNG nor XPM image"
      else
        print_verbose 2 "Installing menu image $ctxdir/$bf"
        mkdir -p "$ctxdir/${size}x${size}"
        cp "$f" "$ctxdir/${size}x${size}/$dbf"  
      fi
    done
  
    if type update-desktop-database 2>/dev/null >/dev/null; then
      print_verbose 2 "Updating icon cache"
      update-desktop-database "$dir"
    else
      print_verbose 1 "No icon cache updated. You may need to log-out an log in again to see new icons"
    fi
  else
    for l in $imglist; do
      size=`echo "$l" | cut -d ':' -f 1-1`
      f=`echo "$l" | cut -d ':' -f 2-`
      bf=`basename $f`
      case "$f" in
        *[pP][nN][gG]|*[xX][pP][mM])
          print_verbose 2 "XDG [$install_mode]: installing menu image $bf => $icon_name"
          "$xdg_desktop_icon" install --noupdate --novendor --context "$install_context" --mode $install_mode --size $size "$f" "$icon_name"
          ;;
          
        *)
          test -f $f && print_verbose 1 "XDG [$install_mode]: Skipping image $bf - is not PNG nor XPM image"
          ;;
      esac
    done
    print_verbose 2 "XDG [$install_mode] updating icon cache"
    "$xdg_desktop_icon" forceupdate --mode $install_mode
  fi

  rm -fR "$tmp_dir"

}

remove_icons() {
  # $1 - mode (user/system)
  # $2 - context (mimetypes, apps)
  # $3 - application (icon) name
  # $4... other images (todo)

  local install_mode="$1"
  local install_context="$2"
  local icon_name="$3"

  print_verbose 4 "Removing [mode=$install_mode, name=$icon_name] $install_context icons: $@"

  xdg_desktop_icon=
  test $disable_xdg = 1 || xdg_desktop_icon=`which xdg-icon-resource 2>/dev/null`
  if [ "X$xdg_desktop_icon" = "X" ]; then
    # no xdg-icon-resource - manually copy desktop files to destination dir
    local dir=`get_dir_based_on_context "$install_mode" "$sys_iconsdir" "$user_iconsdir"`
    local ctxdir="$dir/hicolor/$install_context"
    local deleted=0
    
    print_verbose 2 "XDG icons support disabled - manually removing icons files from $ctxdir"
    if [ -d "$ctxdir" ]; then
      for size in 16 22 24 32 48 64 128; do
        for f in "$ctxdir/${size}x${size}/$icon_name".*; do
          if [ -f "$" ]; then
            print_verbose 2 "Removing image file `basename "$f"`"
            rm -f "$f"
            deleted=1
          fi
        done
      done
  
      if [ "$deleted" = "1" ]; then
        if type update-desktop-database 2>/dev/null >/dev/null; then
          print_verbose 2 "Updating icon cache"
         update-desktop-database "$dir"
        fi
      fi
    fi
  else
    for size in 16 22 24 32 48 64 128; do
      print_verbose 2 "XDG [$install_mode]: removing menu image $icon_name, size $size"
      "$xdg_desktop_icon" uninstall --noupdate --context "$install_context" --mode $install_mode --size $size "$icon_name"
    done
    print_verbose 2 "XDG [$install_mode] updating icon cache"
    "$xdg_desktop_icon" forceupdate --mode $install_mode
  fi
}

install_app_icons() {
  # $1 - context (user/system)
  # $2 - application (icon) name
  # $3 - app main icon image
  # $4... other image files, can contain size suffix (e.g. application-32.png)
  
  install_mode="$1"
  icon_name="$2"
  main_icon="$3"
  shift 3
  
  print_verbose 4 "Installing  [mode=$install_mode] application image $icon_name: $main_icon $@"
 
  install_icons "$install_mode" apps $icon_name "$main_icon" "$@"
  
  # install main image to pixmaps dir
  local dir=`get_dir_based_on_context "$install_mode" "$sys_pixmapsdir" "$user_pixmapsdir"`
  if [ "X$dir" != "X" ]; then
    bf=`basename $main_icon | sed 's/\..*//'`
    dst_main_icon=
    alt_ext=
    case $main_icon in
      *[pP][nN][gG])  dst_main_icon="$bf.png"; altext=.xpm;;
      *[xX][pP][mM])  dst_main_icon="$bf.xpm"; altext=.png;;
      *)
         test -f $main_icon && print_verbose 1 "Skipping image $main_icon - not PNG nor XPM image"
         ;;
    esac
    
    for f; do
      if [ "X$dst_main_icon" = "X`basename $f`" ]; then
        print_verbose 2 "Installing main image $dir/`basename $f`"
        mkdir -p "$dir"
        cp "$f" "$dir/$dst_main_icon"
        dst_main_icon=
      elif [ "X${bf}${altext}" = "X`basename $f`" -o "X`echo $f | grep -v $icon_name | grep -v $bf`" != "X" ]; then
        print_verbose 2 "Installing menu image $dir/`basename $f`"
        mkdir -p "$dir"
        cp "$f" "$dir"
      fi
    done
    
    if [ "X$dst_main_icon" != "X" ]; then
      print_verbose 2 "Installing main image $dir/$dst_main_icon"
      mkdir -p "$dir"
      cp "$main_icon" "$dir/$dst_main_icon"
    fi
  fi
}


remove_app_icons() {
  # $1 - context (user/system)
  # $2 - application (icon) name
  # $3... other images
  
  install_mode="$1"
  shift
  
  print_verbose 4 "Removing [mode=$install_mode] application images: $@"
   
  remove_icons "$install_mode" apps "$@"

  # install main image to pixmaps dir
  local dir=`get_dir_based_on_context "$install_mode" "$sys_pixmapsdir" "$user_pixmapsdir"`

  if [ "X$dir" != "X" ]; then
    for f; do
      bf=`basename $f`
      name=
      case $f in
        *.[pP][nN][gG]|*.[xX][pP][mM])
          name="$dir/$bf"
          ;;
        *.*)
         test -f $f && print_verbose 1 "Skipping image $main_icon - not PNG nor XPM image"
         ;;
        *)
         name=`ls "$dir/$bf".* 2>/dev/null || true`
         ;;
      esac
      
      if [ "X$name" != "X" ]; then
        print_verbose 2 "Removing image $name"
        rm -f $name || true
      fi
    done

    rmdir $dir 2>/dev/null || true
  fi
}



install_mime_icons() {
  # $1 - context (user/system)
  # $2 - mime_types list, can be separated by spaces or :
  # $3 - app main icon image
  # $4... other images if image matches one of mime types, it is used, otherwise main icon file is chosen.
  #      If image contains numeric suffix it is assumed that is the size of image.
  install_mode="$1"
  mime_types="$2"
  main_icon="$3"
  shift 3
  
  print_verbose 4 "Installing [mode=$install_mode] mime-types images $mime_types: $main_icon $@"
  
  test "X$main_icon" = "X" && return 0
 
  # OK - for all mime types
  echo "$mime_types" | grep ':' > /dev/null && mime_types=`split_list $mime_types`
  for mt in $mime_types; do
    mime_type_icon_name=`mime_type_to_icon_name $mt`
    print_verbose 1 "Installing image for $mt mime type"
    install_icons "$install_mode" mimetypes $mime_type_icon_name "$main_icon" "$@"
  done
}

remove_mime_icons() {
  # $1 - context (user/system)
  # $2 - mime_types list, can be separated by spaces or :

  install_mode="$1"
  mime_types="$2"
  
  print_verbose 2 "Removing [mode=$install_mode] mime-types images $mime_types"
  
  # OK - for all mime types
  echo "$mime_types" | grep ':' > /dev/null && mime_types=`split_list $mime_types`
  for mt in $mime_types; do
    mime_type_icon_name=`mime_type_to_icon_name $mt`
    print_verbose 1 "Removing image for $mt mime type"
    remove_icons "$install_mode" mimetypes $mime_type_icon_name 
  done
}


sys_kde4eservicepath=/usr/share/kde4/services:/usr/share/kde/services
user_kde4eservicepath=~/.kde/share/kde4/services:~/.kde4/share/kde4/services:~/.kde4/share/kde4/services/ServiceMenus

# funkcja instaluje menu kontekstowe dla KDE (Konqueror, Dolphin, być może inne)
install_konqueror_ctxmenu() {
  # $1 - install mode (user/system)
  # $2... - source desktop files
  
  install_mode="$1"
  shift

  user_path=$user_kde4eservicepath
  sys_path=$user_kde4eservicepath
  no_kde=1
  if detect_kde; then
    kde4config_path=`kde4-config --path services`
    user_path=`print_first_path_on_list "$kde4config_path"`
    sys_path=`print_last_path_on_list "$kde4config_path"`
    no_kde=0
  fi
  
  local dir=`get_dir_based_on_context "$install_mode" "$sys_path" "$user_path"`
  
  test "$no_kde" = 1 && print_verbose 1 "Warning: No KDE Desktop found, using default directory $dir"

  for f; do
    if test -f "$f"; then
      case "$f" in
        *.desktop)
          print_verbose 1 "Installing KDE Action(s): "`print_kde_actions_names $f`
          print_verbose 2 "Installing KDE Action(s) file: $dir/`basename $f`"
          mkdir -p "$dir"
          cp $f $dir
          ;;
        *)
          print_verbose 1 "`basename $f`: Unrecognized file type, not installing for KDE"
          ;;
      esac
    fi
  done
}


# funkcja usuwa menu kontekstowe dla KDE (Konqueror, Dolphin, być może inne)
remove_konqueror_ctxmenu() {
  # $1 - install mode (user/system)
  # $2... - source desktop files
  
  install_mode="$1"
  shift

  user_path=$user_kde4eservicepath
  sys_path=$user_kde4eservicepath
  if detect_kde; then
    kde4config_path=`kde4-config --path services`
    user_path=`print_first_path_on_list "$kde4config_path"`
    sys_path=`print_last_path_on_list "$kde4config_path"`
  fi
  
  local dir=`get_dir_based_on_context "$install_mode" "$sys_path" "$user_path"`
  
  for f; do
    bf=`basename $f`
    df="$dir/$bf"
    if test -f "$df"; then
      case "$df" in
        *.desktop)
          print_verbose 1 "Removing KDE Action(s): "`print_kde_actions_names $f`
          print_verbose 2 "Removing KDE Action(s) file: $df"
          rm -f "$df"
          ;;
          
        *)
          print_verbose 1 "Removing KDE Action(s): $df: Unrecognized file type, removing anyway"
          ;;
      esac
    fi
    
    # Try to remove empty directory
    rmdir $dir 2>/dev/null || true
  done
}

sys_gconfschemaspath=/etc/gconf/schemas:/usr/share/gconf/schemas
user_gconfschemaspath=~/.gconf

# funkcja instaluje menu kontekstowe dla GNOME - nautilus
install_nautilus_ctxmenu() {
  
  return 0
}


# funkcja usuwa menu kontekstowe dla GNOME - nautilus
remove_nautilus_ctxmenu() {
  return 0
}

sys_thunarconfpath=/etc/xfce/xdg/Thunar
user_thunarconfpath=~/.config/Thunar

# Instalacja menu kontekstowego dla XFCE (Thunar)
install_thunar_ctxmenu() {
 
  install_mode="$1"
  shift

  local dir=`get_dir_based_on_context "$install_mode" "$sys_thunarconfpath" "$user_thunarconfpath"`
  uca_file="$dir/uca.xml"
  
#  if [ -d "$dir" -o $f "$uca_file" ]; then
#    # For user settings, if no directory is present or no uca.xml is present, we need to copy
#    # first system custom actions, otherwise system actions won't be visible
#  fi
#  
#  else
#    mkdir -p "$dir"
#    echo "" > $uca_file
#    echo
#  fi
}


# Usunięcie menu kontekstowego dla XFCE (Thunar)
remove_thunar_ctxmenu() {
  return 0
}

sys_pcmanfmconfpath=/usr/share/file-manager/Actions
user_pcmanfmconfpath=~/.local/share/file-manager/Actions

# Instalacja menu kontekstowego dla Menedżera plików LXDE (PCManFM)
install_pcmanfm_ctxmenu() {
  return 0
}


# Usunięcie menu kontekstowego dla Menedżera plików LXDE (PCManFM)
remove_pcmanfm_ctxmenu() {
  return 0
}

# Remove Actions







# Selected options
action=
defstate=1

disable_xdg=0

mode=

process_menu=
process_mimetypes=
process_icons=
process_mimeicons=
process_ctxmenu=

process_kde=
process_gnome=
process_xfce=
process_lxde=

verbose_level=1

print_verbose() {
  level=$1
  shift
  
  if [ "$level" -le "$verbose_level" ]; then
    echo "$@"
  fi
}


detect_actions() {
  # Menu items
  test "X$process_menu" = "X"      && process_menu=$defstate
  test "X$process_mimetypes" = "X" && process_mimetypes=$defstate
  test "X$process_icons" = "X"     && process_icons=$defstate
  
  test "X$process_mimeicons" = "X" && process_mimeicons=$defstate


  # Context menu
  if [ "$process_ctxmenu" = "1" -o "X$process_ctxmenu" = "X"  ]; then
    if [ "X$process_kde" = "X" ]; then
      process_kde=$defstate
      detect_kde   || detect_kde_fm   || process_kde=0
    fi
  
    if [ "X$process_gnome" = "X" ]; then
      process_gnome=$defstate
      detect_gnome || detect_gnome_fm || process_gnome=0
    fi
  
    if [ "X$process_xfce" = "X" ]; then
      process_xfce=$defstate
      detect_xfce  || detect_xfce_fm  || process_xfce=0
    fi

    if [ "X$process_lxde" = "X" ]; then
      process_lxde=$defstate
      detect_lxde   || detect_lxde_fm   || process_lxde=0
    fi
    
    if [ "X$process_ctxmenu" = "X" ]; then
      process_ctxmenu=0
      test "$process_kde" = "1" -o "$process_gnome" = "1" -o  "$process_xfce" = "1" -o  "$process_lxde" = "1" && process_ctxmenu=1
    fi
    
  else
    test "X$process_kde" = "X"   && process_kde=0
    test "X$process_gnome" = "X" && process_gnome=0
    test "X$process_xfce" = "X"  && process_xfce=0
    test "X$process_lxde" = "X"  && process_lxde=0
  fi
  
  
  
  if [ "$verbose_level" -ge 3 ]; then
    print_verbose 3 "Process menu:       $process_menu"
    print_verbose 3 "Process icons:      $process_icons"
    print_verbose 3 "Process mime types: $process_mimetypes"
    print_verbose 3 "Process mime icons: $process_mimeicons"
    print_verbose 3 "Process ctx menu:   $process_ctxmenu"

    print_verbose 3 "Process KDE:        $process_kde"
    print_verbose 3 "Process GNOME:      $process_gnome"
    print_verbose 3 "Process XFCE:       $process_xfce"
    print_verbose 3 "Process LXDE:       $process_lxde"
  fi
}


do_install() {
  # $1 - context (user/system)
  # $2 - appname
  # $3 - application data
  # $4 - application images
  
  print_verbose 5 "do_install ARGS: $@"
  
  install_mode="$1"
  install_appname="$2"
  install_datadir="$3"
  install_imgdir="$4"
  
  install_mode=`detect_mode "$install_mode"`
  
  test "$process_menu" = "1"      && install_menu_items "$install_mode" $install_datadir/menu/*  $install_datadir/applications/*
  
  test "$process_mimetypes" = "1" && install_mime_types "$install_mode" $install_datadir/mime/* $install_datadir/mimetypes/*
  
  if test "$process_icons" = "1"; then
    if [ -f "$install_imgdir/$install_appname.png" ]; then 
      install_app_icons "$install_mode" $install_appname "$install_imgdir/$install_appname.png" `ls $install_imgdir/* 2>/dev/null | grep -v $install_imgdir/$install_appname.png`
    else
      install_app_icons "$install_mode" $install_appname `ls $install_imgdir/* 2>/dev/null`
    fi
  fi
  
  if test "$process_mimeicons" = "1"; then
    mime_types=`extract_mime_types $install_datadir/mime/* $install_datadir/mimetypes/*`
    if [ "X$mime_types" != "X" ]; then
      if [ -f "$install_imgdir/$install_appname.png" ]; then 
        install_mime_icons "$install_mode" "$mime_types" "$install_imgdir/$install_appname.png" `ls $install_imgdir/* 2>/dev/null | grep -v $install_imgdir/$install_appname.png`
      else
        install_mime_icons "$install_mode" "$mime_types" `ls $install_imgdir/* 2>/dev/null`
      fi
    fi
  fi
  
  # Context menu
  if test "$process_ctxmenu" = "1"; then
     test "$process_kde" = "1"   && install_konqueror_ctxmenu "$install_mode" $install_datadir/kde*/* $install_datadir/konqueror/* $install_datadir/doplhin/*
     test "$process_gnome" = "1" && install_nautilus_ctxmenu
     test "$process_xfce" = "1"  && install_thunar_ctxmenu
     test "$process_lxde" = "1"  && install_pcmanfm_ctxmenu
  fi
}

do_remove() {
  # $1 - context (user/system)
  # $2 - appname
  # $3 - application data
  # $4 - application images

  print_verbose 5 "do_remove ARGS: $@"

  install_mode="$1"
  install_appname="$2"
  install_datadir="$3"
  install_imgdir="$4"
  
  
  install_mode=`detect_mode "$install_mode"`
  
  test "$process_menu" = "1"      && remove_menu_items "$install_mode" $install_datadir/menu/* $install_datadir/applications/*
  
  test "$process_mimetypes" = "1" && remove_mime_types "$install_mode" $install_datadir/mime/* $install_datadir/mimetypes/*
  
  test "$process_icons" = "1"     && remove_app_icons "$install_mode" "$install_appname"
  
  if test "$process_mimeicons" = "1"; then
    mime_types=`extract_mime_types $install_datadir/mime/* $install_datadir/mimetypes/*`
    remove_mime_icons "$install_mode" "$mime_types"
  fi
  
  # Context menu
  if test "$process_ctxmenu" = "1"; then
     test "$process_kde" = "1"   && remove_konqueror_ctxmenu "$install_mode" $install_datadir/kde*/* $install_datadir/konqueror/* $install_datadir/doplhin/*
     test "$process_gnome" = "1" && remove_nautilus_ctxmenu
     test "$process_xfce" = "1"  && remove_thunar_ctxmenu
     test "$process_lxde" = "1"  && remove_pcmanfm_ctxmenu
  fi  
}

# Process command line options
while [ "X$1" != "X" ]; do
  
  case "$1" in
    -h|-help|--help)
       print_help
       exit 0;;

    --version|-V)
       print_version
       exit 0;;

    --install)                  action=install; defstate=1;;
    --install-only)             action=install; defstate=0;;
    --remove)                   action=remove;  defstate=1;;
    --remove-only)              action=remove;  defstate=0;;
    --uninstall)                action=remove;  defstate=1;;
    --uninstall-only)           action=remove;  defstate=0;;
    
    --user)                     mode="user";;
    --system)                   mode="system";;
    
    --verbose|-v)               verbose_level=`expr $verbose_level + 1`;;
    --quiet|-q)                 verbose_level=0;;
    --disable-xdg|-x)           disable_xdg=1;;
    
    --appname)                  APPNAME="$2"; shift;;
    --datadir)                  DATADIR="$2"; shift;;
    --icondir)                  ICONDIR="$2"; shift;;
    
    # Components
    --menu)                     process_menu=1;;
    --no-menu)                  process_menu=0;;
    --auto-menu)                process_menu="";;
    
    --images|--icons)           process_icons=1;;
    --no-images|--no-icons)     process_icons=0;;
    --auto-images|--auto-icons) process_icons="";;
    
    --mime-types)               process_mimetypes=1;;
    --no-mime-types)            process_mimetypes=0;;
    --auto-mime-types)          process_mimetypes="";;

    --mime-icons)               process_mimeicons=1;;
    --no-mime-icons)            process_mimeicons=0;;
    --auto-mime-icons)          process_mimeicons="";;

    --ctx-menu)                 process_ctxmenu=1;;
    --no-ctx-menu)              process_ctxmenu=0;;
    --auto-ctx-menu)            process_ctxmenu="";;

    # Desktop Environments
    --force-kde)                process_kde=1;;
    --no-kde)                   process_kde=0;;
    --auto-kde)                 process_kde="";;

    --force-gnome)              process_gnome=1;;
    --no-gnome)                 process_gnome=0;;
    --auto-gnome)               process_gnome="";;

    --force-xfce)               process_xfce=1;;
    --no-xfce)                  process_xfce=0;;
    --auto-xfce)                process_xfce="";;

    --force-lxde)               process_lxde=1;;
    --no-lxde)                  process_lxde=0;;
    --auto-lxde)                process_lxde="";;


    *)
      print_error "Unrecognized option: $1"
      print_error "type $0 --help to see usage"
      exit 1;    
  esac
  shift
done

case "$action" in
  '')
    print_error "No action specified"
    print_error "type $0 --help to see usage"
    exit 1;;
esac


appdir=`absolute_path "$argv0dir"`

case "$DATADIR" in
  '')    appdata="$appdir/data" ;;
  /*)    appdata="$DATADIR" ;;
  *)     appdata="$appdir/$DATADIR" ;;
esac

case "$ICONDIR" in
  '')    appicons="$appdir/images" ;;
  /*)    appicons="$ICONDIR" ;;
  *)     appicons="$appdir/$ICONDIR" ;;
esac


test "X$APPNAME" = "X" && appname="`basename $appdir`" || appname=$APPNAME

print_verbose 3 "APPNAME: $appname"
print_verbose 3 "DATADIR: $appdata"
print_verbose 3 "ICONDIR: $appicons"


# OK - main script
detect_actions
mode=`detect_mode "$mode"`
if test "$action" = "install"; then
  do_install "$mode" "$appname" "$appdata" "$appicons"
elif test "$action" = "remove"; then
  do_remove  "$mode" "$appname" "$appdata" "$appicons"
else
  print_error "Unknown action $action"
  exit 1
fi
