#!/bin/sh
# Runner script to start application using shared libs from executable directory in first place
# (C) 2014 Enigma Systemy Ochrony Informacji Sp. z o.o.

set -e

me=$0
appname=`basename $me`


PrintError() {
    echo "$@" > /dev/stderr
}

Fatal() {
  PrintError "$@";
  exit 1;
}

# Normalizes path
# Assumes path exists
NormalizePath() {
  if [ ! -e "$1" ]; then
    Fatal "$1: does not exist"
    exit 1;
  fi

  if [ -d "$1" ]; then
    cd "$1" && pwd

  else
    b=`basename $1`
    d=`dirname $1`
    echo `cd "$d" && pwd`/$b

  fi
}

# Function resolves symlink, prints absolute path
ResolveSymLink() {
  # $1 - name to resolve
  name="$1"
  abspath=""
  if [ -L "$name" ]; then
    path=`readlink "$name"`

    case "$path" in
      /*) # Full path
        abspath="$path"
        ;;
      *)  #Relative path
        dir=`dirname "$name"`
        abspath=`NormalizePath "$dir/$path"`
        ;;
    esac
 
  elif [ -e "$name" ]; then
    case "$path" in
      /*) # Full path
        abspath="$path"
        ;;
      *)  #Relative path
        dir=`dirname "$name"`
        abspath=`NormalizePath "$dir/$path"`
        ;;
    esac
  else
    PrintError "$name: Can't find file or directory"
    exit 1;

  fi
  echo $abspath
}


ResolveSymLinksChain() {
  # $1 - name to resolve
  name="$1"
  abspath=`NormalizePath "$name"`
  while true;
  do
    if [ -L "$abspath" ]; then
      abspath=`ResolveSymLink "$name"`
    else
      break;
    fi
  done
  echo $abspath
}



# Resolves symlinks and seaches in PATH if necessary
GetAppLocation() {
  # $1 - app name
  app="$1"
  baseapp=`basename "$app"`
  if [ "X$app" = "X$baseapp" ]; then
    # if appllication is started by only name, search in $PATH
    pathapp=`which "$app" 2>/dev/NULL`
    if [ "X$pathapp" = "X" ]; then
      fatal "Can't find executable $app in \$PATH"
      exit 1;
    fi
    app="$pathapp"
  fi
  
  # Ok path found
  ResolveSymLinksChain "$app"
}


# Find this script location
fullme=`GetAppLocation $me`
test "X$fullme" = "X" && Fatal "Can't find app location" && exit 1
appdir=`dirname $fullme`


#echo "me:     '$me'"
#echo "appname:'$appname'"
#echo "fullme: '$fullme'"
#echo "appdir: '$appdir'"

libpath=$appdir
test -d $appdir/lib && libpath=$appdir/lib
test -d $appdir/../lib && libpath=$appdir/../lib

# Set-up environment
if [ "X$LD_LIBRARY_PATH" = "X" ]; then
  LD_LIBRARY_PATH="$libpath"
else
  LD_LIBRARY_PATH="$libpath:$LD_LIBRARY_PATH"
fi

export LD_LIBRARY_PATH
export QT_QPA_PLATFORM_PLUGIN_PATH="$libpath/platforms"

# Start application
test "$appdir/$appname" = "$me" && appname=`basename $me .sh`

test -x $appdir/$appname-bin && exec ${appdir}/${appname}-bin "$@"
exec ${appdir}/${appname} "$@"

Fatal "Can't execute $appdir/$appname nor $appdir/$appname-bin"
exit 1
